"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.reducer = exports.requireResource = exports.deleteResource = exports.updateResource = exports.readEndpoint = exports.createResource = exports.setAccessToken = exports.hydrateStore = exports.setHeader = exports.setHeaders = exports.setEndpointPath = exports.setEndpointHost = exports.IS_UPDATING = exports.IS_DELETING = exports.__assign = undefined;

var _assign = require("./assign");

var _reduxActions = require("redux-actions");

var _constants = require("./constants");

var _constants2 = _interopRequireDefault(_constants);

var _rxjsImports = require("./rxjs-imports");

var _stateMutation = require("./state-mutation");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var imm = require("object-path-immutable");
var API_SET_ENDPOINT_HOST = _constants2.default.API_SET_ENDPOINT_HOST,
    API_SET_ENDPOINT_PATH = _constants2.default.API_SET_ENDPOINT_PATH,
    API_SET_HEADERS = _constants2.default.API_SET_HEADERS,
    API_SET_HEADER = _constants2.default.API_SET_HEADER,
    API_HYDRATE = _constants2.default.API_HYDRATE,
    API_WILL_CREATE = _constants2.default.API_WILL_CREATE,
    API_CREATED = _constants2.default.API_CREATED,
    API_CREATE_FAILED = _constants2.default.API_CREATE_FAILED,
    API_WILL_READ = _constants2.default.API_WILL_READ,
    API_READ = _constants2.default.API_READ,
    API_READ_FAILED = _constants2.default.API_READ_FAILED,
    API_WILL_UPDATE = _constants2.default.API_WILL_UPDATE,
    API_UPDATED = _constants2.default.API_UPDATED,
    API_UPDATE_FAILED = _constants2.default.API_UPDATE_FAILED,
    API_WILL_DELETE = _constants2.default.API_WILL_DELETE,
    API_DELETED = _constants2.default.API_DELETED,
    API_DELETE_FAILED = _constants2.default.API_DELETE_FAILED;
exports.__assign = _assign.__assign;
var IS_DELETING = exports.IS_DELETING = "IS_DELETING";
var IS_UPDATING = exports.IS_UPDATING = "IS_UPDATING";
var setEndpointHost = exports.setEndpointHost = (0, _reduxActions.createAction)(API_SET_ENDPOINT_HOST);
var setEndpointPath = exports.setEndpointPath = (0, _reduxActions.createAction)(API_SET_ENDPOINT_PATH);
var setHeaders = exports.setHeaders = (0, _reduxActions.createAction)(API_SET_HEADERS);
var setHeader = exports.setHeader = (0, _reduxActions.createAction)(API_SET_HEADER);
var hydrateStore = exports.hydrateStore = (0, _reduxActions.createAction)(API_HYDRATE);
var apiWillCreate = (0, _reduxActions.createAction)(API_WILL_CREATE);
var apiCreated = (0, _reduxActions.createAction)(API_CREATED);
var apiCreateFailed = (0, _reduxActions.createAction)(API_CREATE_FAILED);
var apiWillRead = (0, _reduxActions.createAction)(API_WILL_READ);
var apiRead = (0, _reduxActions.createAction)(API_READ);
var apiReadFailed = (0, _reduxActions.createAction)(API_READ_FAILED);
var apiWillUpdate = (0, _reduxActions.createAction)(API_WILL_UPDATE);
var apiUpdated = (0, _reduxActions.createAction)(API_UPDATED);
var apiUpdateFailed = (0, _reduxActions.createAction)(API_UPDATE_FAILED);
var apiWillDelete = (0, _reduxActions.createAction)(API_WILL_DELETE);
var apiDeleted = (0, _reduxActions.createAction)(API_DELETED);
var apiDeleteFailed = (0, _reduxActions.createAction)(API_DELETE_FAILED);
var setAccessToken = exports.setAccessToken = function (at) {
    return function (dispatch) {
        dispatch(setHeader({ Authorization: "Bearer " + at }));
    };
};
var createResource = exports.createResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillCreate(resource));
        var _a = getState().api.endpoint,
            apiHost = _a.host,
            apiPath = _a.path,
            headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type;
        return (0, _utils.apiRequest)(endpoint, {
            headers: headers,
            method: "POST",
            withCredentials: true,
            body: JSON.stringify({
                data: resource
            })
        }).map(function (json) {
            dispatch(apiCreated(json));
            return json;
        }).catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiCreateFailed(err));
            return _rxjsImports.Observable.throw(err);
        });
    };
};
var readEndpoint = exports.readEndpoint = function (endpoint, _a) {
    var _b = (_a === void 0 ? {} : _a).options,
        options = _b === void 0 ? {
        indexLinks: undefined
    } : _b;
    return function (dispatch, getState) {
        dispatch(apiWillRead(endpoint));
        var _a = getState().api.endpoint,
            apiHost = _a.host,
            apiPath = _a.path,
            headers = _a.headers;
        var apiEndpoint = "" + apiHost + apiPath + "/" + endpoint;
        return (0, _utils.apiRequest)("" + apiEndpoint, {
            headers: headers,
            withCredentials: true
        }).map(function (json) {
            dispatch(apiRead((0, _assign.__assign)({ endpoint: endpoint, options: options }, json)));
            return json;
        }).catch(function (error) {
            var err = error;
            err.endpoint = endpoint;
            dispatch(apiReadFailed(err));
            return _rxjsImports.Observable.throw(err);
        });
    };
};
var updateResource = exports.updateResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillUpdate(resource));
        var _a = getState().api.endpoint,
            apiHost = _a.host,
            apiPath = _a.path,
            headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type + "/" + resource.id;
        return (0, _utils.apiRequest)(endpoint, {
            headers: headers,
            method: "PATCH",
            withCredentials: true,
            body: JSON.stringify({
                data: resource
            })
        }).map(function (json) {
            dispatch(apiUpdated(json));
            return json;
        }).catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiUpdateFailed(err));
            return _rxjsImports.Observable.throw(err);
        });
    };
};
var deleteResource = exports.deleteResource = function (resource) {
    return function (dispatch, getState) {
        dispatch(apiWillDelete(resource));
        var _a = getState().api.endpoint,
            apiHost = _a.host,
            apiPath = _a.path,
            headers = _a.headers;
        var endpoint = "" + apiHost + apiPath + "/" + resource.type + "/" + resource.id;
        return (0, _utils.apiRequest)(endpoint, {
            headers: headers,
            method: "DELETE",
            withCredentials: true
        }).do(function () {
            dispatch(apiDeleted(resource));
        }).catch(function (error) {
            var err = error;
            err.resource = resource;
            dispatch(apiDeleteFailed(err));
            return _rxjsImports.Observable.throw(err);
        });
    };
};
var requireResource = exports.requireResource = function (resourceType, endpoint) {
    if (endpoint === void 0) {
        endpoint = resourceType;
    }
    return function (dispatch, getState) {
        var api = getState().api;
        if (api.hasOwnProperty(resourceType)) {
            return _rxjsImports.Observable.of();
        }
        return dispatch(readEndpoint(endpoint));
    };
};
var reducer = exports.reducer = (0, _reduxActions.handleActions)((_a = {}, _a[API_SET_HEADERS] = function (state, _a) {
    var headers = _a.payload;
    return imm(state).set(["endpoint", "headers"], headers).value();
}, _a[API_SET_HEADER] = function (state, _a) {
    var header = _a.payload;
    var newState = imm(state);
    Object.keys(header).forEach(function (key) {
        newState.set(["endpoint", "headers", key], header[key]);
    });
    return newState.value();
}, _a[API_SET_ENDPOINT_HOST] = function (state, _a) {
    var host = _a.payload;
    return imm(state).set(["endpoint", "host"], host).value();
}, _a[API_SET_ENDPOINT_PATH] = function (state, _a) {
    var path = _a.payload;
    return imm(state).set(["endpoint", "path"], path).value();
}, _a[API_HYDRATE] = function (state, _a) {
    var resources = _a.payload;
    var entities = Array.isArray(resources.data) ? resources.data : [resources.data];
    var newState = (0, _stateMutation.updateOrInsertResourcesIntoState)(state, entities.concat(resources.included || []));
    return imm(newState).value();
}, _a[API_WILL_CREATE] = function (state) {
    return imm(state).set(["isCreating"], state.isCreating + 1).value();
}, _a[API_CREATED] = function (state, _a) {
    var resources = _a.payload;
    var entities = Array.isArray(resources.data) ? resources.data : [resources.data];
    var newState = (0, _stateMutation.updateOrInsertResourcesIntoState)(state, entities.concat(resources.included || []));
    return imm(newState).set("isCreating", state.isCreating - 1).value();
}, _a[API_CREATE_FAILED] = function (state) {
    return imm(state).set(["isCreating"], state.isCreating - 1).value();
}, _a[API_WILL_READ] = function (state) {
    return imm(state).set(["isReading"], state.isReading + 1).value();
}, _a[API_READ] = function (state, _a) {
    var payload = _a.payload;
    var resources = (Array.isArray(payload.data) ? payload.data : [payload.data]).concat(payload.included || []);
    var newState = (0, _stateMutation.updateOrInsertResourcesIntoState)(state, resources);
    var finalState = (0, _stateMutation.addLinksToState)(newState, payload.links, payload.options);
    return imm(finalState).set("isReading", state.isReading - 1).value();
}, _a[API_READ_FAILED] = function (state) {
    return imm(state).set(["isReading"], state.isReading - 1).value();
}, _a[API_WILL_UPDATE] = function (state, _a) {
    var resource = _a.payload;
    var type = resource.type,
        id = resource.id;
    var newState = (0, _stateMutation.ensureResourceTypeInState)(state, type);
    return (0, _stateMutation.setIsInvalidatingForExistingResource)(newState, { type: type, id: id }, IS_UPDATING).set("isUpdating", state.isUpdating + 1).value();
}, _a[API_UPDATED] = function (state, _a) {
    var resources = _a.payload;
    var entities = Array.isArray(resources.data) ? resources.data : [resources.data];
    var newState = (0, _stateMutation.updateOrInsertResourcesIntoState)(state, entities.concat(resources.included || []));
    return imm(newState).set("isUpdating", state.isUpdating - 1).value();
}, _a[API_UPDATE_FAILED] = function (state, _a) {
    var resource = _a.payload.resource;
    var type = resource.type,
        id = resource.id;
    return (0, _stateMutation.setIsInvalidatingForExistingResource)(state, { type: type, id: id }, IS_UPDATING).set("isUpdating", state.isUpdating + 1).value();
}, _a[API_WILL_DELETE] = function (state, _a) {
    var resource = _a.payload;
    var type = resource.type,
        id = resource.id;
    return (0, _stateMutation.setIsInvalidatingForExistingResource)(state, { type: type, id: id }, IS_DELETING).set("isDeleting", state.isDeleting + 1).value();
}, _a[API_DELETED] = function (state, _a) {
    var resource = _a.payload;
    return (0, _stateMutation.removeResourceFromState)(state, resource).set("isDeleting", state.isDeleting - 1).value();
}, _a[API_DELETE_FAILED] = function (state, _a) {
    var resource = _a.payload.resource;
    var type = resource.type,
        id = resource.id;
    return (0, _stateMutation.setIsInvalidatingForExistingResource)(state, { type: type, id: id }, IS_DELETING).set("isDeleting", state.isDeleting + 1).value();
}, _a), {
    isCreating: 0,
    isReading: 0,
    isUpdating: 0,
    isDeleting: 0,
    endpoint: {
        host: null,
        path: null,
        headers: {
            "Content-Type": "application/vnd.api+json",
            Accept: "application/vnd.api+json"
        }
    }
});
var _a;
//# sourceMappingURL=jsonapi.js.map