"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.ensureResourceTypeInState = exports.setIsInvalidatingForExistingResource = exports.updateOrInsertResourcesIntoState = exports.removeResourceFromState = exports.updateOrInsertResource = exports.addLinksToState = exports.makeUpdateReverseRelationship = exports.__assign = undefined;

var _findIndex2 = require("lodash/findIndex");

var _findIndex3 = _interopRequireDefault(_findIndex2);

var _find2 = require("lodash/find");

var _find3 = _interopRequireDefault(_find2);

var _assign = require("./assign");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var equal = require("deep-equal");
var imm = require("object-path-immutable");
var pluralize = require("pluralize");
exports.__assign = _assign.__assign;
var makeUpdateReverseRelationship = exports.makeUpdateReverseRelationship = function (resource, relationship, newRelation) {
    if (newRelation === void 0) {
        newRelation = {
            type: resource.type,
            id: resource.id
        };
    }
    return function (foreignResources) {
        var idx = (0, _findIndex3.default)(foreignResources, function (item) {
            return item.id === relationship.data.id;
        });
        if (idx === -1) {
            return foreignResources;
        }
        var _a = [1, 2].map(function (i) {
            return pluralize(resource.type, i);
        }),
            singular = _a[0],
            plural = _a[1];
        var relCase = (0, _find3.default)([singular, plural], function (r) {
            return (0, _utils.hasOwnProperties)(foreignResources[idx], ["relationships", r]);
        });
        if (!relCase) {
            return foreignResources;
        }
        var relPath = ["relationships", relCase, "data"];
        var idxRelPath = [idx].concat(relPath);
        var immutableForeingResources = imm(foreignResources);
        if (!(0, _utils.hasOwnProperties)(foreignResources[idx], relPath)) {
            return immutableForeingResources.push(idxRelPath, newRelation).value();
        }
        var foreignResourceRel = foreignResources[idx].relationships[relCase].data;
        if (newRelation && Array.isArray(foreignResourceRel) && (0, _findIndex3.default)(~foreignResourceRel, function (rel) {
            return rel.id === newRelation.id && rel.type === newRelation.type;
        }) || newRelation && foreignResourceRel && foreignResourceRel.id === newRelation.id && foreignResourceRel.type === newRelation.type) {
            return foreignResources;
        } else if (Array.isArray(foreignResourceRel) && !newRelation) {
            var relIdx = (0, _findIndex3.default)(foreignResourceRel, function (item) {
                return item.id === resource.id;
            });
            if (foreignResourceRel[relIdx]) {
                var deletePath = [idx, "relationships", singular, "data", relIdx];
                return imm(foreignResources).del(deletePath).value();
            }
            return foreignResources;
        }
        if (relCase === singular) {
            return immutableForeingResources.set(idxRelPath, newRelation).value();
        }
        return immutableForeingResources.push(idxRelPath, newRelation).value();
    };
};
var stateContainsResource = function (state, resource) {
    var updatePath = [resource.type, "data"];
    if ((0, _utils.hasOwnProperties)(state, updatePath)) {
        return (0, _findIndex3.default)(state[resource.type].data, function (item) {
            return item.id === resource.id;
        }) > -1;
    }
    return false;
};
var addLinksToState = exports.addLinksToState = function (state, links, options) {
    if (options === undefined || options.indexLinks === undefined) {
        return state;
    }
    var indexLinkName = options.indexLinks;
    var newState = imm.set(state, "links." + indexLinkName, links);
    return newState;
};
var updateOrInsertResource = exports.updateOrInsertResource = function (state, resource) {
    if (typeof resource !== "object") {
        return state;
    }
    var newState = state;
    var updatePath = [resource.type, "data"];
    if (stateContainsResource(state, resource)) {
        var resources = state[resource.type].data;
        var idx = (0, _findIndex3.default)(resources, function (item) {
            return item.id === resource.id;
        });
        if (resource.relationships) {
            var relationships = {};
            for (var relationship in resource.relationships) {
                if (!resource.relationships[relationship].data) {
                    relationships[relationship] = resources[idx].relationships[relationship];
                }
            }
            (0, _assign.__assign)(resource.relationships, relationships);
        }
        if (!equal(resources[idx], resource)) {
            newState = imm.set(newState, updatePath.concat(idx), resource);
        }
    } else {
        newState = imm.push(newState, updatePath, resource);
    }
    var rels = resource.relationships;
    if (!rels) {
        return newState;
    }
    Object.keys(rels).forEach(function (relKey) {
        if (!(0, _utils.hasOwnProperties)(rels[relKey], ["data", "type"])) {
            return;
        }
        var entityPath = [rels[relKey].data.type, "data"];
        if (!(0, _utils.hasOwnProperties)(newState, entityPath)) {
            return;
        }
        var updateReverseRelationship = makeUpdateReverseRelationship(resource, rels[relKey]);
        newState = imm.set(newState, entityPath, updateReverseRelationship(newState[rels[relKey].data.type].data));
    });
    return newState;
};
var removeResourceFromState = exports.removeResourceFromState = function (state, resource) {
    var index = (0, _findIndex3.default)(state[resource.type].data, function (e) {
        return e.id === resource.id;
    });
    var path = [resource.type, "data", index];
    var entityRelationships = resource.relationships || {};
    return Object.keys(entityRelationships).reduce(function (newState, key) {
        if (!resource.relationships[key].data) {
            return newState;
        }
        var entityPath = [resource.relationships[key].data.type, "data"];
        if ((0, _utils.hasOwnProperties)(state, entityPath)) {
            var updateReverseRelationship = makeUpdateReverseRelationship(resource, resource.relationships[key], null);
            return newState.set(entityPath, updateReverseRelationship(state[resource.relationships[key].data.type].data));
        }
        return newState;
    }, imm(state).del(path));
};
var updateOrInsertResourcesIntoState = exports.updateOrInsertResourcesIntoState = function (state, resources) {
    return resources.reduce(updateOrInsertResource, state);
};
var setIsInvalidatingForExistingResource = exports.setIsInvalidatingForExistingResource = function (state, _a, value) {
    var type = _a.type,
        id = _a.id;
    if (value === void 0) {
        value = null;
    }
    var idx = (0, _findIndex3.default)(state[type].data, function (e) {
        return e.id === id && e.type === type;
    });
    var updatePath = [type, "data", idx, "isInvalidating"];
    return value === null ? imm(state).del(updatePath) : imm(state).set(updatePath, value);
};
var ensureResourceTypeInState = exports.ensureResourceTypeInState = function (state, type) {
    var path = [type, "data"];
    return (0, _utils.hasOwnProperties)(state, [type]) ? state : imm(state).set(path, []).value();
};
//# sourceMappingURL=state-mutation.js.map